<?php

if (!class_exists('PHP_Token_NAME_QUALIFIED')) {
    class PHP_Token_NAME_QUALIFIED extends PHP_Token
    {
    }
}

if (!class_exists('PHP_Token_ATTRIBUTE')) {
    class PHP_Token_ATTRIBUTE extends PHP_Token
    {
    }
}

if (!class_exists('PHP_Token_AMPERSAND_NOT_FOLLOWED_BY_VAR_OR_VARARG')) {
    class PHP_Token_AMPERSAND_NOT_FOLLOWED_BY_VAR_OR_VARARG extends PHP_Token
    {
    }
}

if (!class_exists('PHP_Token_AMPERSAND_FOLLOWED_BY_VAR_OR_VARARG')) {
    class PHP_Token_AMPERSAND_FOLLOWED_BY_VAR_OR_VARARG extends PHP_Token
    {
    }
}

if (!class_exists('PHP_Token_NAME_FULLY_QUALIFIED')) {
    class PHP_Token_NAME_FULLY_QUALIFIED extends PHP_Token
    {
    }
}
