html {
}

body {
    background-color: #fff;
    background-image: url(../images/header.png);
    background-position: 0 0;
    background-repeat: repeat-x;

	margin: 0 auto;

	font-family: sans-serif;
	font-size: 12px;

	background-color: #ffffff;
	color: #2e3436;
}

h1 {
	margin-bottom: 25px;
	padding: 0;
}

h1 a {
    display: block;
	text-indent: -9999px;
	overflow: hidden;

	height: 110px;

	background-color: transparent;
	background-image: url(../images/logo.png);
	background-repeat: no-repeat;
	background-position: left top;
}

p {
	font-family: sans-serif;
	font-size: 1.1em;
	line-height: 1.5em;
	color: #2e3436;
}

code {
	font-family: monospace;
}

a {
	color: #4e9a06;
}

img {
	border: none;
}

div.content {
    position: relative;

	padding: 0 200px 2em 0;
	margin-top: -2em;
    width: 600px !important;
}

div.footer {
    padding: 0.4em 0;

    border-top: 1px solid #888a85;
    border-bottom: 1px solid #888a85;

    font-size: 0.9em;

    background-color: #eeeeec;
}

.twitter-follow {
	background-color: #1b95e0;
	border-radius: 3px;
	color: #fff;
	display: inline-block;
	font-size: 0.8em;
	font-style: normal;
	height: 22px;
	padding: 0 8px 1px 6px;
	text-decoration: none;
}

.twitter-follow > i {
	background: transparent url(data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20viewBox%3D%220%200%2072%2072%22%3E%3Cpath%20fill%3D%22none%22%20d%3D%22M0%200h72v72H0z%22%2F%3E%3Cpath%20class%3D%22icon%22%20fill%3D%22%23fff%22%20d%3D%22M68.812%2015.14c-2.348%201.04-4.87%201.744-7.52%202.06%202.704-1.62%204.78-4.186%205.757-7.243-2.53%201.5-5.33%202.592-8.314%203.176C56.35%2010.59%2052.948%209%2049.182%209c-7.23%200-13.092%205.86-13.092%2013.093%200%201.026.118%202.02.338%202.98C25.543%2024.527%2015.9%2019.318%209.44%2011.396c-1.125%201.936-1.77%204.184-1.77%206.58%200%204.543%202.312%208.552%205.824%2010.9-2.146-.07-4.165-.658-5.93-1.64-.002.056-.002.11-.002.163%200%206.345%204.513%2011.638%2010.504%2012.84-1.1.298-2.256.457-3.45.457-.845%200-1.666-.078-2.464-.23%201.667%205.2%206.5%208.985%2012.23%209.09-4.482%203.51-10.13%205.605-16.26%205.605-1.055%200-2.096-.06-3.122-.184%205.794%203.717%2012.676%205.882%2020.067%205.882%2024.083%200%2037.25-19.95%2037.25-37.25%200-.565-.013-1.133-.038-1.693%202.558-1.847%204.778-4.15%206.532-6.774z%22%2F%3E%3C%2Fsvg%3E) no-repeat 0 0;
	display: inline-block;
	height: 14px;
	position: relative;
	top: 2px;
	width: 14px;
}

.mastodon-follow {
	background-color: #17063B;
	border-radius: 3px;
	color: #fff;
	display: inline-block;
	font-size: 0.8em;
	font-style: normal;
	height: 22px;
	padding: 0 8px 1px 6px;
	text-decoration: none;
}
.mastodon-follow > i {
	background: transparent url("data:image/svg+xml,%3Csvg viewBox='0 0 75 79' fill='none' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath d='M73.8393 17.4898C72.6973 9.00165 65.2994 2.31235 56.5296 1.01614C55.05 0.797115 49.4441 0 36.4582 0H36.3612C23.3717 0 20.585 0.797115 19.1054 1.01614C10.5798 2.27644 2.79399 8.28712 0.904997 16.8758C-0.00358524 21.1056 -0.100549 25.7949 0.0682394 30.0965C0.308852 36.2651 0.355538 42.423 0.91577 48.5665C1.30307 52.6474 1.97872 56.6957 2.93763 60.6812C4.73325 68.042 12.0019 74.1676 19.1233 76.6666C26.7478 79.2728 34.9474 79.7055 42.8039 77.9162C43.6682 77.7151 44.5217 77.4817 45.3645 77.216C47.275 76.6092 49.5123 75.9305 51.1571 74.7385C51.1797 74.7217 51.1982 74.7001 51.2112 74.6753C51.2243 74.6504 51.2316 74.6229 51.2325 74.5948V68.6416C51.2321 68.6154 51.2259 68.5896 51.2142 68.5661C51.2025 68.5426 51.1858 68.522 51.1651 68.5058C51.1444 68.4896 51.1204 68.4783 51.0948 68.4726C51.0692 68.4669 51.0426 68.467 51.0171 68.4729C45.9835 69.675 40.8254 70.2777 35.6502 70.2682C26.7439 70.2682 24.3486 66.042 23.6626 64.2826C23.1113 62.762 22.7612 61.1759 22.6212 59.5646C22.6197 59.5375 22.6247 59.5105 22.6357 59.4857C22.6466 59.4609 22.6633 59.4391 22.6843 59.422C22.7053 59.4048 22.73 59.3929 22.7565 59.3871C22.783 59.3813 22.8104 59.3818 22.8367 59.3886C27.7864 60.5826 32.8604 61.1853 37.9522 61.1839C39.1768 61.1839 40.3978 61.1839 41.6224 61.1516C46.7435 61.008 52.1411 60.7459 57.1796 59.7621C57.3053 59.7369 57.431 59.7154 57.5387 59.6831C65.4861 58.157 73.0493 53.3672 73.8178 41.2381C73.8465 40.7606 73.9184 36.2364 73.9184 35.7409C73.9219 34.0569 74.4606 23.7949 73.8393 17.4898Z' fill='url(%23paint0_linear_549_34)'/%3E%3Cpath d='M61.2484 27.0263V48.114H52.8916V27.6475C52.8916 23.3388 51.096 21.1413 47.4437 21.1413C43.4287 21.1413 41.4177 23.7409 41.4177 28.8755V40.0782H33.1111V28.8755C33.1111 23.7409 31.0965 21.1413 27.0815 21.1413C23.4507 21.1413 21.6371 23.3388 21.6371 27.6475V48.114H13.2839V27.0263C13.2839 22.7176 14.384 19.2946 16.5843 16.7572C18.8539 14.2258 21.8311 12.926 25.5264 12.926C29.8036 12.926 33.0357 14.5705 35.1905 17.8559L37.2698 21.346L39.3527 17.8559C41.5074 14.5705 44.7395 12.926 49.0095 12.926C52.7013 12.926 55.6784 14.2258 57.9553 16.7572C60.1531 19.2922 61.2508 22.7152 61.2484 27.0263Z' fill='white'/%3E%3Cdefs%3E%3ClinearGradient id='paint0_linear_549_34' x1='37.0692' y1='0' x2='37.0692' y2='79' gradientUnits='userSpaceOnUse'%3E%3Cstop stop-color='%236364FF'/%3E%3Cstop offset='1' stop-color='%23563ACC'/%3E%3C/linearGradient%3E%3C/defs%3E%3C/svg%3E");
	display: inline-block;
	height: 14px;
	position: relative;
	top: 4px;
	width: 14px;
}

div.header {
	padding: 0 0 2em 0;

	color: #eeeeef;
    border-bottom: 1px solid #888a85;

	background-color: #eeeeec;
}

div.clear {
    clear: both;
}

.viewport {
	margin: 0px auto;
	width: 800px;
}

/* Forms */

fieldset {
}

label {
	display: block;
	width: 150px;
	text-align: right;
	clear: both;
	float: left;
	padding: 1px 5px 1px 1px;
}

input,
textarea {
	width: 440px;
	float: none;
	float: right;

	border: 1px solid #555753;
	background-color: #ffffff;
	color: #2e3436;
}

input[type="submit"] {
	width: 200px;

	border: 1px solid #555753;
	background-color: #ffffff;
}

input[name="url"] {
	/* Catching spam bots */
	display: none;
}

/* Lists */

dl dt {
    display: block;
    float: left;

    width: 60px;
}

dl dd {
    margin-left: 60px;
}

/* Tabels */

table {
    border: 1px solid #EEEEEC;
    border-collapse: collapse;
}

table th, table td {
    padding: 0.5em;
}

table th {
    background-color: #FCFCFC;
}

table td, table th {
    border-width: 1px 0 0 1px;
    border-style: solid;
    border-color: #EEEEEC;

    vertical-align: top;
}

table td p.cell a {
/*    color: #2E3436;*/
/*    font-weight: bold;*/
    white-space: nowrap;
    text-decoration: none;
}

table td p.cell a:hover {
    color: #4E9A06;
    text-decoration: underline;
}

div.content table p.cell {
    margin: 0 0 0.3em 0;

    line-height: 1.1em;
    text-align: left;
}

/* Navigation */

h2.navigation {
    display: none;
}

ul.navigation {
    background-image: url(../images/puzzle.png);
    background-position: 320px 20px;
    background-repeat: no-repeat;
    
    font-size: 14px;
	list-style-type: none;
	margin: 25px auto 10px auto;
	padding: 5px 0px;
    width: 800px;
}

ul.navigation li ul {
	padding-left: 20px;
	list-style-type: none;
}

ul.navigation li {
    background-image: url(../images/menu-entry-underline.png);
    background-position: bottom left;
    background-repeat: no-repeat;

	font-weight: normal;
}

ul.navigation li.requested {
	/*font-weight: bold;*/
}

ul.navigation li a {
    background-image: url(../images/menu-entry-icon.png);
    background-position: 0 50%;
    background-repeat: no-repeat;

	color: #2e3436;
	display: block;
    padding: 5px 0;
    width: 250px;
    
	text-decoration: none;
    text-indent: 20px;
}

ul.navigation li.requested a {
    background-image: url(../images/menu-entry-requested-icon.png);
}

ul.navigation li a:hover {
	display: block;
	color: #2e3436;
	text-decoration: underline;
}

/* Navigation */

div.sub-navigation {
    position: relative;
    overflow: visible;
    
    
    height: 0;
    width: 100%;

    z-index: 20;
}

div.sub-navigation ul {
    list-style-type: none;
    padding: 0;
}

div.sub-navigation ul li {
    float: left;

    padding: 0;
    margin: 0;
}

div.sub-navigation ul li.requested {
    font-weight: bold;
}

div.sub-navigation ul li a {
    background: transparent none bottom left no-repeat;

    display: block;
    margin: 0.2em 2.5em 0 0;
    padding: 0 0.5em;

    color: #2e3436;

    font-size: 1.1em;
    line-height: 1.6em;
    
    text-decoration: none;
}

div.sub-navigation ul li.requested a {
    background-image: url(../images/sub-menu-entry-underline-requested.png);
    font-weight: normal;
}

div.sub-navigation ul li a:hover {
    background-image: url(../images/sub-menu-entry-underline-hover.png);
}
div.sub-navigation ul li.requested a:hover {
    background-image: url(../images/sub-menu-entry-underline-requested.png);
}

/* Search box */

div#search {
	margin: 2px;
	padding: 5px 0px;

	width: 165px;
	clear: left;
	float: left;
}

div#search h2 {
	margin: 0px 0px 5px 0px;
	font-size: 14px;
}

div#search form {
	margin: 10px;
	padding: 0px;
}

div#search fieldset {
	border: 1px solid #555753;
	background-color: #ffffff;

	margin: 0px;
	padding: 0px;

	width: 125px;
}

div#search label,
div#search legend {
	display: none;
}

div#search input {
	float: none;
	border: none;
}

div#search input#search_phrase {
	width: 105px;
}

div#search input[type=submit] {
	width: 15px;
	height: 18px;

	text-indent: 100px;
	overflow: hidden;

	background-image: url( "/images/search.png" );
	background-repeat: no-repeat;
	background-position: 0px 2px;
}

/* Search result */

ul.search {
	padding: 0px 0px 0px 30px;
	margin: 0px;

	list-style-type: circle;
}

ul.search li {
	margin: 10px 0px;
	border-bottom: 1px solid #d3d7cf;
}

ul.search li span.comment {
	display: block;
	padding: 0px 2px;

	text-align: right;
	text-decoration: none;
}

ul.search li h3 {
	margin: 0px 0px 3px 0px;
	font-size: 14px;
}

ul.search li h4 {
	margin: 0px 0px 3px 0px;
	font-size: 10px;
	font-weight: normal;
	color: #555753;
}

/* Formats */

div.formats {
	float: left;
}

div.formats a {
	text-decoration: none;
}

div.formats a:hover {
	text-decoration: underline;
}

/* License */

div.license {
    float: right;
}

div.license a {
	text-decoration: none;
}

div.license a:hover {
	text-decoration: underline;
}

/* Changelog and Revision */

div.revision {
    display: inline;
    position: relative;
   
	cursor: help;
}

div.revision ul {
    display: none;
}

div.revision:hover ul {
	display: block;

	position: absolute;
	left: -300px;
    bottom: 25px;
	width: 400px;

	list-style-type: circle;
	margin: 0px;
	padding: 2px 2px 2px 15px;

	border: 1px solid #555753;
	font-size: 0.8em;

	background-color: #ffffff;
}

div.revision ul li {
	margin: 1px 0px;
	padding: 0px;
}

div.revision ul li p {
	margin: 1px 0px;
	padding: 0px;
}

/* Content */

h2 {
	margin: 0 0 2.0em 0;
	font-size: 1.6em;
}

div.image {
	margin: 0px 0px 5px 5px;
	padding: 1px 2px;

	font-size: 10px;
	color: #babdb6;
}

div.content p {
	text-align: justify;
}
div.content p.prev, div.content p.next {
    float: left;
    width: 299px;
    margin-top: 0;
}
div.content p.next {
    text-align: right;
}
div.content p.prev a:before {
    content: "« ";
}
div.content p.next a:after {
    content: " »";
}

div.content li p {
	padding: 0;
	margin: 0;
}

a#cloudbees {
    position: absolute;
    left: -160px;
    bottom: 3em;
}

div.image img {
	display: block;
	margin: 0px 0px 1px 0px;
	border: 1px solid #babdb6;
}
div.left {
    float: left;
}

div.toc h3 {
	font-style: italic;
        font-size: 1.0em;

	display: none;

	padding-bottom: 0;
	margin-bottom: 0.2em;
}

div.toc li p {
	line-height: 1.5em;
}

div.toc ul {
	margin: 0;
	padding: 0 0 0 2em;
}

div.toc ul li {
	margin: 0;
	padding: 1px;
}

div.toc ul li ul {
	padding: 0px 0px 0px 20px;
}

div.toc p {
	padding: 0px;
	margin: 0px;
}

code.block {
    white-space: pre;
    display: block;
    background-color: #eeeeef;

    padding: 5px 10px;
    margin: 5px 0;

    overflow: auto;
}

code.shell {
    border: 2px solid #2E3436;
    background-color: #000;

    color: #fff;
    font-weight: bold;
}

code.inline {
    display: inline;
    font-family: monospace;
}

span.scroll {
	display: block;
	clear: both;

	text-align: center;
}

span.scroll a {
	text-decoration: none;
}

/* Blog */

ul.blog {
	padding: 0px;
	margin: 0px;

	list-style-type: none;
}

ul.blog li {
	margin: 10px 0px;
}

ul.blog li span.comment {
	display: block;
	padding: 0px 2px;

	font-size: 10px;
	text-align: right;
	text-decoration: none;
}

ul.blog li a {
	text-decoration: none;
}

ul.blog li h2 {
    display: none;
}

ul.blog li h3 {
    margin-bottom: 0.2em;
}

ul.blog li h3 a {
	font-size: 1.3em;
    color: #2e3436;
}

ul.blog li h3 a:hover {
	text-decoration: underline;
}

ul.blog li h4 {
	margin: 0px 0px 3px 0px;
	font-size: 1.1em;
	font-weight: normal;
	color: #888a85;
}

ul.blog li p, ul.blog li code {
	margin-left: 1.5em;
}

ul.comments {
	padding: 0px 20px;
	margin: 0px;

	list-style-type: none;
}

ul.comments li {
	margin: 5px;
	border: 1px solid #babdb6;
	padding: 2px;
}

ul.comments li h4 {
	margin: 0px 0px 2px 0px;
	padding: 0px;

	font-size: 12px;
	font-weight: normal;
	color: #555753;
}

ul.comments li p {
	margin: 0px;
	padding: 0px;

	font-style: italic;
}

ul.comments li a.permalink {
	display: block;
	padding: 0px 2px;

	font-size: 10px;
	text-align: right;
	text-decoration: none;
}

/* Gallery */

ul.gallery {
    padding: 0px;
    margin: 0px;

    width: 900px !important;
}

ul.gallery li {
    float: left;
    display: block;
    margin: 20px;
}

ul.gallery li h3,
ul.gallery li h4 {
	display: none;
}

dl.gallery dt {
	width: 70px;
	float: left;
	clear: left;

	font-weight: bold;
}

dl.gallery dd {
	float: left;
    width: 150px;
}

img.full {
	margin: 5px auto;
	clear: both;
}

